/*
 * Copyright 2019 the original author or authors.
 *
 * Licensed under the LGPL, Version 3.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at  http://www.gnu.org/licenses/lgpl-3.0.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.jn.sqlhelper.dialect.internal;

import com.jn.sqlhelper.dialect.internal.limit.LimitOffsetLimitHandler;

/**
 * https://pingcap.com/docs/dev/reference/sql/statements/select/
 * https://www.pingcap.com/docs-cn/sql/dml/
 * <p>
 * supports two style：
 * 1) limit $offset, $limit
 * 2) limit $limit offset $offset
 * <p>
 * we use 2)
 */
public class TiDBDialect extends AbstractDialect {
    public TiDBDialect() {
        super();
        setLimitHandler(new LimitOffsetLimitHandler().setSupportLockInModeClause(true));
    }

    @Override
    public boolean isSupportsLimit() {
        return true;
    }

    @Override
    public boolean isSupportsLimitOffset() {
        return true;
    }

    @Override
    public boolean isBindLimitParametersInReverseOrder() {
        return true;
    }
}
